*     Subroutines are used for reading and writing RMC (or MD/MC) 
*     configuration files:
*
*     read_rmc_config   } general reading and writing routines
*     write_rmc_config  }
*
*     read_config       } simpler routines more appropriate
*     write_config      } for atomic systems
*
***********************************************************************

      subroutine read_rmc_config(title,ngen,ntried,nacc,nsaved,
     - max_molecules,max_moltypes,max_sites,nvar,
     - truncated,vectors,
     - nmol,nmoltypes,ni,nsites,sites,
     - neuler,centres,ichan)

      real          centres(nvar,max_molecules)
      real          sites(3,max_sites,max_moltypes)
      real          vectors(3,3)
      integer       ni(max_moltypes),nsites(max_moltypes)
      character*(*) title
      logical truncated,space
*
*     Reads a configuration data file
*
*     Check format of file
*     Also have to allow for possibility of initial space
*     (Fortran carriage control) which may or may not occur at
*     beginning of each line
*
      read (ichan,'(a)') title
      space=.false.
      if (title(1:1).eq.' ') space=.true.
      if (space) title=title(2:len(title))
      if (title(1:10).ne.'(Version 3') then
         write (*,*) 'Configuration file has wrong format'
         stop
      endif
*
      read (ichan,'(a)') title
      if (space) title=title(2:len(title))
      read (ichan,*) ngen,ntried,nacc
      read (ichan,*) nsaved
      read (ichan,*) nmol
      read (ichan,*) nmoltypes
      read (ichan,*) nsites_max
      read (ichan,*) neuler
*
      if (nmol.gt.max_molecules) then
         write (*,*) 'Configuration contains ',nmol,' particles'
         write (*,*) 'Arrays are only large enough for ',max_molecules
         stop
      endif
      if (nmoltypes.gt.max_moltypes) then
         write (*,*) 'Configuration contains ',nmoltypes,
     -     ' types of particle'
         write (*,*) 'Arrays are only large enough for ',max_moltypes
         stop
      endif
      if (nsites_max.gt.max_sites) then
         write (*,*) 'Configuration contains molecules with',nsites_max,
     -    'atomic sites'
         write (*,*) 'Arrays are only large enough for ',max_sites
         stop
      endif
      if (neuler+3.gt.nvar) then
         write (*,*) 'Configuration specifies ',neuler,' Euler angles'
         write (*,*) 'Arrays are only large enough for ',nvar-3
         stop
      endif
* 
      read (ichan,*)
      read (ichan,*) truncated
      read (ichan,*)
      read (ichan,*) vectors
*
      do itype=1,nmoltypes
         read (ichan,*) ni(itype)
         read (ichan,*) nsites(itype)
         read (ichan,*) ((sites(i,isite,itype),i=1,3),
     -    isite=1,nsites(itype))
      enddo
*
      do i=1,nmol
         read (ichan,*) (centres(j,i),j=1,3+neuler)
      enddo
      return
      end

***********************************************************************

      subroutine write_rmc_config(title,ngen,ntried,nacc,nsaved,
     - max_sites,nvar,
     - truncated,vectors,
     - nmol,nmoltypes,ni,nsites,sites,
     - neuler,centres,ichan)
      real          centres(nvar,*)
      real          sites(3,max_sites,*)
      real          vectors(3,3)
      integer       ni(*),nsites(*)
      character*(*) title
      logical truncated
*
*     Writes a configuration data file
*
      write (ichan,*) '(Version 3 format configuration file)'
      write (ichan,*) title
*
      write (ichan,*) 
      write (ichan,'(1x,3i10,a)') ngen,ntried,nacc,
     - ' moves generated, tried, accepted'
      write (ichan,'(1x,i10,20x,a)') nsaved,' configurations saved'
      write (ichan,*) 
      write (ichan,'(1x,i10,a)') nmol,' molecules (of all types)'
      write (ichan,'(1x,i10,a)') nmoltypes,' types of molecule'
      nsites_max=1
      do i=1,nmoltypes
         nsites_max=max(nsites_max,nsites(i))
      enddo
      write (ichan,'(1x,i10,a)') nsites_max,
     - ' is the largest number of atoms in a molecule'
      write (ichan,'(1x,i10,a)') neuler,' Euler angles are provided'
*
      write (ichan,*)
      if (truncated) then
         write (ichan,'(1x,l10,a)') truncated,
     -    ' (Box is truncated octahedral)'
      else
         write (ichan,'(1x,l10,a)') truncated,
     -    ' (Box is not truncated octahedral)'
      endif
      write (ichan,'(11x,a)') ' Defining vectors are:'
      write (ichan,'(3(11x,3(1x,f10.6)/))') vectors
*
      do itype=1,nmoltypes
         write (ichan,'(1x,i10,a,i2)') ni(itype),
     -     ' molecules of type ',itype
         write (ichan,'(1x,i10,a)') nsites(itype),' atomic sites'
         write (ichan,'(100(11x,3(1x,f10.6)/))')
     -    ((sites(i,isite,itype),i=1,3),isite=1,nsites(itype))
      enddo
*
      do i=1,nmol
         write (ichan,'(3(1x,f10.7),3(1x,f10.8))') 
     -    (centres(j,i),j=1,3+neuler)
      enddo
      return
      end

************************************************************************

      subroutine read_config(title,ngen,ntried,nacc,nsaved,
     - max_atoms,max_types,
     - truncated,vectors,
     - n,ntypes,ni,
     - atoms,ichan)
      real          atoms(3,max_atoms)
      real          vectors(3,3)
      integer       ni(*)
      real          sites(3,1,100)
      integer       nsites(100)
      character*(*) title
      logical truncated
*
      call read_rmc_config(title,ngen,ntried,nacc,nsaved,
     - max_atoms,max_types,1,3,truncated,vectors,
     - n,ntypes,ni,nsites,sites,neuler,atoms,ichan)
*
      return
      end

***********************************************************************

      subroutine write_config(title,ngen,ntried,nacc,nsaved,
     - truncated,vectors,n,ntypes,ni,atoms,ichan)
      real          atoms(3,*)
      real          vectors(3,3)
      integer       ni(*)
      real          sites(3,1,100)
      integer       nsites(100)
      character*(*) title
      logical truncated
*
      do i=1,ntypes
         nsites(i)=1
         sites(1,1,i)=0.
         sites(2,1,i)=0.
         sites(3,1,i)=0.
      enddo
      call write_rmc_config(title,ngen,ntried,nacc,nsaved,
     - 1,3,truncated,vectors,n,ntypes,ni,nsites,sites,
     - 0,atoms,ichan)
*
      return
      end
