*      Contains the following machine dependent routines: 
*        rmc_init        initialises clock and random number generator
*        rmc_datafile    returns name of data file to be read
*        rmc_time        calculates CPU time used
*        rmc_rerun       passes RERUN flag to operating system
*        open_read_ext   opens a file for reading
*        open_write      opens a file for writing 
*        real_to_hex     converts real*4 to IEEE hex string
*        hex_to_real     converts IEEE hex string to real*4
*
*-----------------------------------------------------------------------
*
*     Subroutine rmc_data must return the name of the data file to
*     be read by the programme. If possible this will be supplied
*     as a parameter on the command line that invokes the programme
*
      subroutine rmc_datafile(file)
	use msflib
	integer*2 n1
      character*(*) file
	n1 = 1
      call getarg(n1,file)
      return
      end
*
*-----------------------------------------------------------------------
*
*     Subroutine rmc_init must set the seed for the random number 
*     generator and obtain the current cpu clock reading in minutes
*
      subroutine rmc_init(iseed,time_start)
	use portlib
	real*8 time_start
      iseed=int(secnds(0.0))*2+1
      time_start=real(rtc()/60.)
      return
      end
*
*-----------------------------------------------------------------------
*
*     Subroutine rmc_time, given the initial cpu clock reading, must 
*     return the cpu time used (in minutes). If possible should return
*     a negative value if less than 1 minute remains.
*
      subroutine rmc_time(time_start,time_used)
      use portlib
	real*8 time_start,time_used
	real*8 time_start8
	time_start8=time_start
      time_used=real(rtc()/60.-time_start8)
      return
      end
*
*-----------------------------------------------------------------------
*
*     Subroutine rmc_rerun, can be used to return the rerun flag to
*     the operating system, if desired.
*
      subroutine rmc_rerun(rerun)
      logical rerun
      if (rerun) call exit(10)
      return
      end
*
*-----------------------------------------------------------------------
*
*     Subroutine open_read_ext opens a file for reading, filling in the
*     supplied extension if none is supplied with the file name.
*
      subroutine open_read_ext(unit,file,extension)
      integer unit
      character*(*) file,extension,temp*80
      temp=file
      i=index(file,'.')
      l=len(file)
      do while (file(l:l).eq.' ')
         l=l-1
      enddo
      if (i.eq.0) temp=file(1:l)//extension
      open (unit,file=temp,status='old',action='read')
      return
      end
*
*-----------------------------------------------------------------------
*
*     Subroutine open_write opens a file for writing, filling in the
*     supplied extension if none is supplied with the file name.
*     Allows use of system specific facilities of open statement.
*
      subroutine open_write(unit,file,extension)
      integer unit
      character*(*) file,extension,temp*80
      temp=file
      i=index(file,'.')
      l=len(file)
      do while (file(l:l).eq.' ')
         l=l-1
      enddo
      if (i.eq.0) temp=file(1:l)//extension
      open (unit,file=temp,status='old',err=10)
      close(unit,status='delete')
10    open (unit,file=temp,status='new')
      return
      end
*
*-----------------------------------------------------------------------
*
*     Function real_to_hex converts a 32 bit real value to an
*     8 character hex string describing its internal representation.
*     If intermediate files are to be portable this must be
*     converted to IEEE form.
*
*     Assume storage is in Convex native format rather than IEEE.
*     This means we must shift exponent by 2
*
      character*8 function real_to_hex(r)
      character*8 temp
      write (temp,'(z8)') r/4.
      real_to_hex=temp
      return
      end
*
*-----------------------------------------------------------------------
*     Function hex_to_real converts an 8 character hex string in IEEE 
*     format to a 32 bit real value.
*
      real function hex_to_real(hex,error)
      character hex*(*)
      logical error
      error=.false.
      read (hex,'(z8)',end=10,err=10) hex_to_real
      hex_to_real=hex_to_real*4.
      return
10    error=.true.
      return
      end
